@@ -56,10 +56,10 @@ We proceed with hash joins in the following stages:
 from functools import wraps, partial
 import warnings
 
-from toolz import merge_sorted, unique, first
+from tlz import merge_sorted, unique, first
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_dtype_equal
+from pandas.api.types import is_dtype_equal, is_categorical_dtype
 
 from ..base import tokenize, is_dask_collection
 from ..highlevelgraph import HighLevelGraph
@@ -80,7 +80,12 @@ from .core import (
 from .io import from_pandas
 from . import methods
 from .shuffle import shuffle, rearrange_by_divisions
-from .utils import strip_unknown_categories, is_series_like, asciitable, PANDAS_GT_0230
+from .utils import (
+    strip_unknown_categories,
+    is_series_like,
+    asciitable,
+    is_dataframe_like,
+)
 
 
 def align_partitions(*dfs):
@@ -210,7 +215,16 @@ def require(divisions, parts, required=None):
 ###############################################################
 
 
-required = {"left": [0], "right": [1], "inner": [0, 1], "outer": []}
+required = {
+    "left": [0],
+    "leftsemi": [0],
+    "leftanti": [0],
+    "right": [1],
+    "inner": [0, 1],
+    "outer": [],
+}
+allowed_left = ("inner", "left", "leftsemi", "leftanti")
+allowed_right = ("inner", "right")
 
 
 def merge_chunk(lhs, *args, **kwargs):
@@ -324,7 +338,7 @@ def single_partition_join(left, right, **kwargs):
     meta = left._meta_nonempty.merge(right._meta_nonempty, **kwargs)
     kwargs["empty_index_dtype"] = meta.index.dtype
     name = "merge-" + tokenize(left, right, **kwargs)
-    if left.npartitions == 1 and kwargs["how"] in ("inner", "right"):
+    if left.npartitions == 1 and kwargs["how"] in allowed_right:
         left_key = first(left.__dask_keys__())
         dsk = {
             (name, i): (apply, merge_chunk, [left_key, right_key], kwargs)
@@ -338,7 +352,7 @@ def single_partition_join(left, right, **kwargs):
         else:
             divisions = [None for _ in right.divisions]
 
-    elif right.npartitions == 1 and kwargs["how"] in ("inner", "left"):
+    elif right.npartitions == 1 and kwargs["how"] in allowed_left:
         right_key = first(right.__dask_keys__())
         dsk = {
             (name, i): (apply, merge_chunk, [left_key, right_key], kwargs)
@@ -475,11 +489,12 @@ def merge(
         )
 
     # Single partition on one side
+    # Note that cudf supports "leftsemi" and "leftanti" joins
     elif (
         left.npartitions == 1
-        and how in ("inner", "right")
+        and how in allowed_right
         or right.npartitions == 1
-        and how in ("inner", "left")
+        and how in allowed_left
     ):
         return single_partition_join(
             left,
@@ -719,7 +734,7 @@ def _concat_compat(frames, left, right):
     if PANDAS_GT_100:
         # join_axes removed
         return (pd.concat, frames, 0, "outer", False, None, None, None, False, False)
-    elif PANDAS_GT_0230:
+    else:
         # (axis, join, join_axis, ignore_index, keys, levels, names, verify_integrity, sort)
         # we only care about sort, to silence warnings.
         return (
@@ -735,9 +750,6 @@ def _concat_compat(frames, left, right):
             False,
             False,
         )
-    else:
-        columns = get_unsorted_columns([left, right])
-        return (concat_and_unsort, frames, columns)
 
 
 def merge_asof_indexed(left, right, **kwargs):
@@ -924,8 +936,24 @@ def stack_partitions(dfs, divisions, join="outer"):
     for df in dfs:
         # dtypes of all dfs need to be coherent
         # refer to https://github.com/dask/dask/issues/4685
+        # and https://github.com/dask/dask/issues/5968.
+        if is_dataframe_like(df):
+
+            shared_columns = df.columns.intersection(meta.columns)
+            needs_astype = [
+                col
+                for col in shared_columns
+                if df[col].dtype != meta[col].dtype
+                and not is_categorical_dtype(df[col].dtype)
+            ]
+
+            if needs_astype:
+                # Copy to avoid mutating the caller inplace
+                df = df.copy()
+                df[needs_astype] = df[needs_astype].astype(meta[needs_astype].dtypes)
+
         if is_series_like(df) and is_series_like(meta):
-            if not df.dtype == meta.dtype and str(df.dtype) != "category":
+            if not df.dtype == meta.dtype and not is_categorical_dtype(df.dtype):
                 df = df.astype(meta.dtype)
         else:
             pass  # TODO: there are other non-covered cases here
