@@ -3,7 +3,7 @@ from operator import getitem
 import uuid
 import warnings
 
-import toolz
+import tlz as toolz
 import numpy as np
 import pandas as pd
 
