@@ -1,6 +1,5 @@
 import warnings
 
-import dask
 import dask.dataframe as dd
 import numpy as np
 import pandas as pd
@@ -8,7 +7,7 @@ import pandas as pd
 from dask.base import compute_as_if_collection
 from dask.dataframe._compat import tm
 from dask.dataframe.core import _Frame
-from dask.dataframe.methods import concat, concat_kwargs
+from dask.dataframe.methods import concat
 from dask.dataframe.multi import (
     align_partitions,
     merge_indexed_dataframes,
@@ -22,7 +21,6 @@ from dask.dataframe.utils import (
     make_meta,
     has_known_categories,
     clear_known_categories,
-    PANDAS_GT_0230,
 )
 
 import pytest
@@ -529,7 +527,7 @@ def test_indexed_concat(join):
     B = pd.DataFrame({"x": [10, 20, 40, 50, 60, 80]}, index=[1, 2, 4, 5, 6, 8])
     b = dd.repartition(B, [1, 2, 5, 8])
 
-    expected = pd.concat([A, B], axis=0, join=join, **concat_kwargs)
+    expected = pd.concat([A, B], axis=0, join=join, sort=False)
     result = concat_indexed_dataframes([a, b], join=join)
     assert_eq(result, expected)
 
@@ -560,10 +558,7 @@ def test_concat(join):
     )
     ddf3 = dd.from_pandas(pdf3, 2)
 
-    if PANDAS_GT_0230:
-        kwargs = {"sort": False}
-    else:
-        kwargs = {}
+    kwargs = {"sort": False}
 
     for (dd1, dd2, pd1, pd2) in [(ddf1, ddf2, pdf1, pdf2), (ddf1, ddf3, pdf1, pdf3)]:
 
@@ -585,26 +580,36 @@ def test_concat(join):
         assert_eq(result, expected)
 
 
-@pytest.mark.parametrize("join", ["inner", "outer"])
-def test_concat_different_dtypes(join):
+@pytest.mark.parametrize(
+    "value_1, value_2",
+    [
+        (1.0, 1),
+        (1.0, "one"),
+        # See https://github.com/dask/dask/issues/5968 and
+        # https://github.com/pandas-dev/pandas/issues/32934
+        pytest.param(1.0, pd.to_datetime("1970-01-01"), marks=pytest.mark.xfail),
+        (1, "one"),
+        (1, pd.to_datetime("1970-01-01")),
+        ("one", pd.to_datetime("1970-01-01")),
+    ],
+)
+def test_concat_different_dtypes(value_1, value_2):
     # check that the resulting dataframe has coherent dtypes
-    # refer to https://github.com/dask/dask/issues/4685
-    pdf1 = pd.DataFrame(
-        {"x": [1, 2, 3, 4, 6, 7], "y": list("abcdef")}, index=[1, 2, 3, 4, 6, 7]
-    )
-    ddf1 = dd.from_pandas(pdf1, 2)
-    pdf2 = pd.DataFrame(
-        {"x": [1.0, 2.0, 3.0, 4.0, 6.0, 7.0], "y": list("abcdef")},
-        index=[8, 9, 10, 11, 12, 13],
-    )
-    ddf2 = dd.from_pandas(pdf2, 2)
+    # refer to https://github.com/dask/dask/issues/4685 and
+    # https://github.com/dask/dask/issues/5968
+    df_1 = pd.DataFrame({"x": [value_1]})
+    df_2 = pd.DataFrame({"x": [value_2]})
+    df = pd.concat([df_1, df_2], axis=0)
 
-    expected = pd.concat([pdf1, pdf2], join=join)
-    result = dd.concat([ddf1, ddf2], join=join)
-    assert_eq(expected, result)
+    pandas_dtype = df["x"].dtype
 
-    dtypes_list = dask.compute([part.dtypes for part in result.to_delayed()])
-    assert len(set(map(str, dtypes_list))) == 1  # all the same
+    ddf_1 = dd.from_pandas(df_1, npartitions=1)
+    ddf_2 = dd.from_pandas(df_2, npartitions=1)
+    ddf = dd.concat([ddf_1, ddf_2], axis=0)
+
+    dask_dtypes = list(ddf.map_partitions(lambda x: x.dtypes).compute())
+
+    assert dask_dtypes == [pandas_dtype, pandas_dtype]
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
@@ -724,6 +729,58 @@ def test_merge(how, shuffle):
     #         pd.merge(A, B, left_index=True, right_on='y'))
 
 
+@pytest.mark.parametrize("parts", [(3, 3), (3, 1), (1, 3)])
+@pytest.mark.parametrize("how", ["leftsemi", "leftanti"])
+@pytest.mark.parametrize(
+    "engine",
+    [
+        "cudf",
+        pytest.param(
+            "pandas",
+            marks=pytest.mark.xfail(
+                reason="Pandas does not support leftsemi or leftanti"
+            ),
+        ),
+    ],
+)
+def test_merge_tasks_semi_anti_cudf(engine, how, parts):
+    if engine == "cudf":
+        # NOTE: engine == "cudf" requires cudf/dask_cudf,
+        # will be skipped by non-GPU CI.
+
+        cudf = pytest.importorskip("cudf")
+        dask_cudf = pytest.importorskip("dask_cudf")
+
+    emp = pd.DataFrame(
+        {
+            "emp_id": np.arange(101, stop=106),
+            "name": ["John", "Tom", "Harry", "Rahul", "Sakil"],
+            "city": ["Cal", "Mum", "Del", "Ban", "Del"],
+            "salary": [50000, 40000, 80000, 60000, 90000],
+        }
+    )
+    skills = pd.DataFrame(
+        {
+            "skill_id": [404, 405, 406, 407, 408],
+            "emp_id": [103, 101, 105, 102, 101],
+            "skill_name": ["Dask", "Spark", "C", "Python", "R"],
+        }
+    )
+
+    if engine == "cudf":
+        emp = cudf.from_pandas(emp)
+        skills = cudf.from_pandas(skills)
+        dd_emp = dask_cudf.from_cudf(emp, npartitions=parts[0])
+        dd_skills = dask_cudf.from_cudf(skills, npartitions=parts[1])
+    else:
+        dd_emp = dd.from_pandas(emp, npartitions=parts[0])
+        dd_skills = dd.from_pandas(skills, npartitions=parts[1])
+
+    expect = emp.merge(skills, on="emp_id", how=how).sort_values(["emp_id"])
+    result = dd_emp.merge(dd_skills, on="emp_id", how=how).sort_values(["emp_id"])
+    assert_eq(result, expect, check_index=False)
+
+
 def test_merge_tasks_passes_through():
     a = pd.DataFrame({"a": [1, 2, 3, 4, 5, 6, 7], "b": [7, 6, 5, 4, 3, 2, 1]})
     b = pd.DataFrame({"c": [1, 2, 3, 4, 5, 6, 7], "d": [7, 6, 5, 4, 3, 2, 1]})
@@ -1402,7 +1459,7 @@ def test_concat2():
         pdcase = [_c.compute() for _c in case]
 
         with warnings.catch_warnings(record=True) as w:
-            expected = pd.concat(pdcase, **concat_kwargs)
+            expected = pd.concat(pdcase, sort=False)
 
         ctx = FutureWarning if w else None
 
@@ -1418,7 +1475,7 @@ def test_concat2():
             assert set(result.dask) == set(dd.concat(case).dask)
 
         with warnings.catch_warnings(record=True) as w:
-            expected = pd.concat(pdcase, join="inner", **concat_kwargs)
+            expected = pd.concat(pdcase, join="inner", sort=False)
 
         ctx = FutureWarning if w else None
 
@@ -1448,7 +1505,7 @@ def test_concat3():
     ddf3 = dd.from_pandas(pdf3, 2)
 
     with warnings.catch_warnings(record=True) as w:
-        expected = pd.concat([pdf1, pdf2], **concat_kwargs)
+        expected = pd.concat([pdf1, pdf2], sort=False)
 
     ctx = FutureWarning if w else None
 
@@ -1466,7 +1523,7 @@ def test_concat3():
         )
 
     with warnings.catch_warnings(record=True) as w:
-        expected = pd.concat([pdf1, pdf2, pdf3], **concat_kwargs)
+        expected = pd.concat([pdf1, pdf2, pdf3], sort=False)
 
     ctx = FutureWarning if w else None
 
@@ -1523,12 +1580,11 @@ def test_concat4_interleave_partitions():
         pdcase = [c.compute() for c in case]
 
         assert_eq(
-            dd.concat(case, interleave_partitions=True),
-            pd.concat(pdcase, **concat_kwargs),
+            dd.concat(case, interleave_partitions=True), pd.concat(pdcase, sort=False)
         )
         assert_eq(
             dd.concat(case, join="inner", interleave_partitions=True),
-            pd.concat(pdcase, join="inner", **concat_kwargs),
+            pd.concat(pdcase, join="inner", sort=False),
         )
 
     msg = "'join' must be 'inner' or 'outer'"
@@ -1588,7 +1644,7 @@ def test_concat5():
             # some cases will raise warning directly from pandas
             assert_eq(
                 dd.concat(case, interleave_partitions=True),
-                pd.concat(pdcase, **concat_kwargs),
+                pd.concat(pdcase, sort=False),
             )
 
         assert_eq(
@@ -1821,31 +1877,27 @@ def test_append2():
     meta = make_meta({"b": "i8", "c": "i8"})
     ddf3 = dd.DataFrame(dsk, "y", meta, [None, None])
 
-    assert_eq(ddf1.append(ddf2), ddf1.compute().append(ddf2.compute(), **concat_kwargs))
-    assert_eq(ddf2.append(ddf1), ddf2.compute().append(ddf1.compute(), **concat_kwargs))
+    assert_eq(ddf1.append(ddf2), ddf1.compute().append(ddf2.compute(), sort=False))
+    assert_eq(ddf2.append(ddf1), ddf2.compute().append(ddf1.compute(), sort=False))
 
     # different columns
-    assert_eq(ddf1.append(ddf3), ddf1.compute().append(ddf3.compute(), **concat_kwargs))
-    assert_eq(ddf3.append(ddf1), ddf3.compute().append(ddf1.compute(), **concat_kwargs))
+    assert_eq(ddf1.append(ddf3), ddf1.compute().append(ddf3.compute(), sort=False))
+    assert_eq(ddf3.append(ddf1), ddf3.compute().append(ddf1.compute(), sort=False))
 
     # Dask + pandas
     assert_eq(
-        ddf1.append(ddf2.compute()),
-        ddf1.compute().append(ddf2.compute(), **concat_kwargs),
+        ddf1.append(ddf2.compute()), ddf1.compute().append(ddf2.compute(), sort=False)
     )
     assert_eq(
-        ddf2.append(ddf1.compute()),
-        ddf2.compute().append(ddf1.compute(), **concat_kwargs),
+        ddf2.append(ddf1.compute()), ddf2.compute().append(ddf1.compute(), sort=False)
     )
 
     # different columns
     assert_eq(
-        ddf1.append(ddf3.compute()),
-        ddf1.compute().append(ddf3.compute(), **concat_kwargs),
+        ddf1.append(ddf3.compute()), ddf1.compute().append(ddf3.compute(), sort=False)
     )
     assert_eq(
-        ddf3.append(ddf1.compute()),
-        ddf3.compute().append(ddf1.compute(), **concat_kwargs),
+        ddf3.append(ddf1.compute()), ddf3.compute().append(ddf1.compute(), sort=False)
     )
 
 
