@@ -1,24 +1,20 @@
-from __future__ import absolute_import, division, print_function
-
 import operator
 import types
 import uuid
 import warnings
+from collections.abc import Iterator
 
-try:
-    from cytoolz import curry, concat, unique, merge
-except ImportError:
-    from toolz import curry, concat, unique, merge
+from tlz import curry, concat, unique, merge
 
 from . import config, threaded
 from .base import is_dask_collection, dont_optimize, DaskMethodsMixin
 from .base import tokenize as _tokenize
-from .compatibility import apply, Iterator, is_dataclass, dataclass_fields
+from .compatibility import is_dataclass, dataclass_fields
 
 from .core import quote
 from .context import globalmethod
 from .optimization import cull
-from .utils import funcname, methodcaller, OperatorMethodMixin, ensure_dict
+from .utils import funcname, methodcaller, OperatorMethodMixin, ensure_dict, apply
 from .highlevelgraph import HighLevelGraph
 
 __all__ = ["Delayed", "delayed"]
